<?php

namespace Drupal\age_exposed_filter\Plugin\views\filter;

use Drupal\views\Plugin\views\filter\NumericFilter;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;

/**
 * Age filter.
 *
 * @ingroup views_filter_handlers
 *
 * @ViewsFilter("age_filter")
 */
class AgeFilter extends NumericFilter {

  /**
   * Override parent method to adapt 'between' operations for ages comparison.
   *
   * The initial solution was to pass variables into the addWhereExpression()
   * using 'arguments' array as the third parameter.
   * But it caused the following issue: if we add two 'Age filters'
   * then expression only uses 'arguments' of the first one.
   */
  public function opBetween($field) {
    // If a user input non numeric value, then don't display any results.
    if (!is_numeric($this->value['min']) or !is_numeric($this->value['max'])) {
      $this->query->addWhereExpression($this->options['group'], 'FALSE');
      return;
    }

    $age_min = $this->value['min'];
    $age_max = $this->value['max'];

    $current_date = new DrupalDateTime();
    $current_date->setTimezone(new \DateTimezone(DateTimeItemInterface::STORAGE_TIMEZONE));

    $current_year = $current_date->format('Y');
    $current_month = $current_date->format('m');
    $current_day = $current_date->format('d');

    $birthday_year = $this->query->getDateFormat($field, 'Y', FALSE);
    $birthday_month = $this->query->getDateFormat($field, 'm', FALSE);
    $birthday_day = $this->query->getDateFormat($field, 'd', FALSE);

    $current_year_minus_age_min = ($current_year - $age_min);
    $current_year_minus_age_max = ($current_year - $age_max);
    $current_year_minus_age_min_minus_one = ($current_year - $age_min - 1);
    $current_year_minus_age_max_minus_one = ($current_year - $age_max - 1);
    $current_month = $current_month;
    $current_day = $current_day;

    if ($this->operator == 'between') {

      $exp = '(';
        $exp .= '(';
          $exp .= "($birthday_month < $current_month)";
          $exp .= " AND ($birthday_year <= $current_year_minus_age_min)";
        $exp .= ')';
        $exp .= ' OR ';
        $exp .= '(';
          $exp .= "($birthday_month > $current_month)";
          $exp .= " AND ($birthday_year <= $current_year_minus_age_min_minus_one)";
        $exp .= ')';
        $exp .= ' OR ';
        $exp .= '(';
          $exp .= "($birthday_month = $current_month)";
          $exp .= " AND ($birthday_day > $current_day)";
          $exp .= " AND ($birthday_year <= $current_year_minus_age_min_minus_one)";
        $exp .= ')';
        $exp .= ' OR ';
        $exp .= '(';
          $exp .= "($birthday_month = $current_month)";
          $exp .= " AND ($birthday_day <= $current_day)";
          $exp .= " AND ($birthday_year <= $current_year_minus_age_min)";
        $exp .= ')';
      $exp .= ')';

      $exp .= ' AND ';

      $exp .= '(';
        $exp .= '(';
          $exp .= "($birthday_month < $current_month)";
          $exp .= " AND ($birthday_year >= $current_year_minus_age_max)";
        $exp .= ')';
        $exp .= ' OR ';
        $exp .= '(';
          $exp .= "($birthday_month > $current_month)";
          $exp .= " AND ($birthday_year >= $current_year_minus_age_max_minus_one)";
        $exp .= ')';
        $exp .= ' OR ';
        $exp .= '(';
          $exp .= "($birthday_month = $current_month)";
          $exp .= " AND ($birthday_day > $current_day)";
          $exp .= " AND ($birthday_year >= $current_year_minus_age_max_minus_one)";
        $exp .= ')';
        $exp .= ' OR ';
        $exp .= '(';
          $exp .= "($birthday_month = $current_month)";
          $exp .= " AND ($birthday_day <= $current_day)";
          $exp .= " AND ($birthday_year >= $current_year_minus_age_max)";
        $exp .= ')';
      $exp .= ')';
    }
    elseif ($this->operator == 'not between') {

      $exp = '(';
        $exp .= '(';
          $exp .= "($birthday_month < $current_month)";
          $exp .= " AND ($birthday_year > $current_year_minus_age_min)";
        $exp .= ')';
        $exp .= ' OR ';
        $exp .= '(';
          $exp .= "($birthday_month > $current_month)";
          $exp .= " AND ($birthday_year > $current_year_minus_age_min_minus_one)";
        $exp .= ')';
        $exp .= ' OR ';
        $exp .= '(';
          $exp .= "($birthday_month = $current_month)";
          $exp .= " AND ($birthday_day > $current_day)";
          $exp .= " AND ($birthday_year > $current_year_minus_age_min_minus_one)";
        $exp .= ')';
        $exp .= ' OR ';
        $exp .= '(';
          $exp .= "($birthday_month = $current_month)";
          $exp .= " AND ($birthday_day <= $current_day)";
          $exp .= " AND ($birthday_year > $current_year_minus_age_min)";
        $exp .= ')';
      $exp .= ')';

      $exp .= ' OR ';

      $exp .= '(';
        $exp .= '(';
          $exp .= "($birthday_month < $current_month)";
          $exp .= " AND ($birthday_year < $current_year_minus_age_max)";
        $exp .= ')';
        $exp .= ' OR ';
        $exp .= '(';
          $exp .= "($birthday_month > $current_month)";
          $exp .= " AND ($birthday_year < $current_year_minus_age_max_minus_one)";
        $exp .= ')';
        $exp .= ' OR ';
        $exp .= '(';
          $exp .= "($birthday_month = $current_month)";
          $exp .= " AND ($birthday_day > $current_day)";
          $exp .= " AND ($birthday_year < $current_year_minus_age_max_minus_one)";
        $exp .= ')';
        $exp .= ' OR ';
        $exp .= '(';
          $exp .= "($birthday_month = $current_month)";
          $exp .= " AND ($birthday_day <= $current_day)";
          $exp .= " AND ($birthday_year < $current_year_minus_age_max)";
        $exp .= ')';
      $exp .= ')';
    }

    $this->query->addWhereExpression($this->options['group'], $exp);
  }

  /**
   * Override parent method to adapt simple operations for ages comparison.
   *
   * The initial solution was to pass variables into the addWhereExpression()
   * using 'arguments' array as the third parameter.
   * But it caused the following issue: if we add two 'Age filters'
   * then expression only uses 'arguments' of the first one.
   */
  public function opSimple($field) {
    // If a user input non numeric value, then don't display any results.
    if (!is_numeric($this->value['value'])) {
      $this->query->addWhereExpression($this->options['group'], 'FALSE');
      return;
    }

    switch ($this->operator) {
      case '<':
        $operator = '>';
        break;

      case '>':
        $operator = '<';
        break;

      case '<=':
        $operator = '>=';
        break;

      case '>=':
        $operator = '<=';
        break;

      default:
        $operator = $this->operator;
    }

    $age = $this->value['value'];
    $current_date = new DrupalDateTime();
    $current_date->setTimezone(new \DateTimezone(DateTimeItemInterface::STORAGE_TIMEZONE));

    $current_year = $current_date->format('Y');
    $current_month = $current_date->format('m');
    $current_day = $current_date->format('d');

    $birthday_year = $this->query->getDateFormat($field, 'Y', FALSE);
    $birthday_month = $this->query->getDateFormat($field, 'm', FALSE);
    $birthday_day = $this->query->getDateFormat($field, 'd', FALSE);

    $current_year_minus_age = ($current_year - $age);
    $current_year_minus_age_minus_one = ($current_year - $age - 1);
    $current_month = $current_month;
    $current_day = $current_day;

    $exp = '(';
      $exp .= "($birthday_month < $current_month)";
      $exp .= " AND ($birthday_year $operator $current_year_minus_age)";
    $exp .= ')';
    $exp .= ' OR ';
    $exp .= '(';
      $exp .= "($birthday_month > $current_month)";
      $exp .= " AND ($birthday_year $operator $current_year_minus_age_minus_one)";
    $exp .= ')';
    $exp .= ' OR ';
    $exp .= '(';
      $exp .= "($birthday_month = $current_month)";
      $exp .= " AND ($birthday_day > $current_day)";
      $exp .= " AND ($birthday_year $operator $current_year_minus_age_minus_one)";
    $exp .= ')';
    $exp .= ' OR ';
    $exp .= '(';
      $exp .= "($birthday_month = $current_month)";
      $exp .= " AND ($birthday_day <= $current_day)";
      $exp .= " AND ($birthday_year $operator $current_year_minus_age)";
    $exp .= ')';

    $this->query->addWhereExpression($this->options['group'], $exp);
  }

  /**
   * Override the method to exclude the 'regular_expression' operator.
   */
  public function operators() {
    $operators = [
      '<' => [
        'title' => $this->t('Is less than'),
        'method' => 'opSimple',
        'short' => $this->t('<'),
        'values' => 1,
      ],
      '<=' => [
        'title' => $this->t('Is less than or equal to'),
        'method' => 'opSimple',
        'short' => $this->t('<='),
        'values' => 1,
      ],
      '=' => [
        'title' => $this->t('Is equal to'),
        'method' => 'opSimple',
        'short' => $this->t('='),
        'values' => 1,
      ],
      '!=' => [
        'title' => $this->t('Is not equal to'),
        'method' => 'opSimple',
        'short' => $this->t('!='),
        'values' => 1,
      ],
      '>=' => [
        'title' => $this->t('Is greater than or equal to'),
        'method' => 'opSimple',
        'short' => $this->t('>='),
        'values' => 1,
      ],
      '>' => [
        'title' => $this->t('Is greater than'),
        'method' => 'opSimple',
        'short' => $this->t('>'),
        'values' => 1,
      ],
      'between' => [
        'title' => $this->t('Is between'),
        'method' => 'opBetween',
        'short' => $this->t('between'),
        'values' => 2,
      ],
      'not between' => [
        'title' => $this->t('Is not between'),
        'method' => 'opBetween',
        'short' => $this->t('not between'),
        'values' => 2,
      ],
    ];

    // If the definition allows for the empty operator, add it.
    if (!empty($this->definition['allow empty'])) {
      $operators += [
        'empty' => [
          'title' => $this->t('Is empty (NULL)'),
          'method' => 'opEmpty',
          'short' => $this->t('empty'),
          'values' => 0,
        ],
        'not empty' => [
          'title' => $this->t('Is not empty (NOT NULL)'),
          'method' => 'opEmpty',
          'short' => $this->t('not empty'),
          'values' => 0,
        ],
      ];
    }

    return $operators;
  }

}
